package FlexPay;

use strict;
use warnings;
use base 'Exporter';

use Digest::SHA1 qw( sha1_hex );
use Params::Validate qw(:all);
use URI;
use URI::QueryParam;
use Carp;
use utf8;

our @EXPORT_OK = qw( 
    get_signature 
    get_status_URL
    get_purchase_URL 
    validate_signature
    $SIGNATURE_KEY
);

our $SIGNATURE_KEY = 'SET_THIS_TO_YOUR_SIGNATURE_KEY'; # Set this to your signature key
# The signature key can be found in the Verotel control center

our $STATUS_URL = 'https://secure.verotel.com/status/purchase';
our $PURCHASE_URL = 'https://secure.verotel.com/order/purchase';

=head2 get_signature( $signatureKey, %params ) 

Returns sha1_hex signature for given purchase parameters.

Signature is SHA-1 hash as hexadecimal number generated from $signatureKey followed
by parameters joined with colon (:). Parameters ("$key=$value") are 
alphabeticaly orderered by their keys. 

=head3 Example: 

    get_signature('aaB', shopID => '123', custom1 => 'xyz', custom2 => undef , wrong => 'bla' );

    uses signature() 
    
    returns generated lower case converted SHA-1 string for "aaB:custom1=xyz:custom2=:shopID=123" 

=cut

sub get_signature {
    my $signatureKey = shift;
    my %params = @_;
    return signature($signatureKey, %params);
}


=head2 validate($signatureKey, %params) 

Returns true only if signature generated from parameters matches the input signature 

Function DOES NOT filter params! Parameters must be correct match 

=head3 Example:
    
    RETURNS TRUE:
    
    validate_signature('aaB', shopID => 123, saleID => 345, version => 1, signature => '5d8f0efa55398812b5fa68215966086a10368198');
    
    RETURN FALSE:

    - if you add unallowed params 
    - if you omit or have wrong signature
    - if you omit params
    - if some of the params have wrong values
    - if you give wrong signatureKey

=cut 

sub validate_signature {
    my ($signatureKey, %params) = @_;
    my $sign1  = lc(delete $params{signature}); 
    my $sign2  = signature($signatureKey, %params); 
    return ($sign1 eq $sign2) ? 1 : 0;
}


=head2 get_purchase_URL( $signature_signatureKey, %params ) 

Return URL for purchase with signed parameters (Only purchase parameters are 
signed, but all given parameters are added to URL) 

=head3 Example:

    get_purchase_URL('aaB', shopID => 123, saleID => 345, version => 1);

    RETURNS:

        'https://secure.verotel.com/order/purchase?saleID=345&shopID=123&version=1&signature=5d8f0efa55398812b5fa68215966086a10368198';

=cut 

sub get_purchase_URL {
    my ($signatureKey, %params) = @_;
    if (!$signatureKey) {
       return "ERROR no signatureKey given";
    };
    if (!%params) {
       return "ERROR no params given";
    };
    return generate_URL($PURCHASE_URL, $signatureKey, %params);
}


=head2 get_status_URL( $signature_signatureKey, %params ) 

Return URL for status with signed parameters (Only purchase parameters are 
signed, but all given parameters are added to URL) 


=head3 Example:

    get_status_URL('aaB', shopID => 123, saleID => 345, version => 1);

    RETURNS:

        'https://secure.verotel.com/status/purchase?saleID=345&shopID=123&version=1&signature=5d8f0efa55398812b5fa68215966086a10368198';

=cut 

sub get_status_URL {
    my ($signatureKey, %params) = @_;
    if (!$signatureKey) {
       return "ERROR no signatureKey given";
    };
    if (!%params) {
       return "ERROR no params given";
    };
    return generate_URL($STATUS_URL, $signatureKey, %params);
}


################ PRIVATE METHODS ##########################

=head2 generate_URL( $baseURL, $signatureKey, %params ) 

Generate and return URL with signed parameters (Only purchase parameters are 
signed, but all given parameters are added to URL) 

    uses filter_params() - filters out wrong, unallowed, or empty params

=cut 

sub generate_URL {
    my ($baseURL, $signatureKey, %params) = (@_);

    # remove undefined values: 
    %params = map { defined($params{$_}) 
                            ? ($_ => $params{$_}) 
                            : () 
                    } sort keys %params;

    my $url         = new URI($baseURL);
    %params = filter_params(%params);
    my $signature   = get_signature($signatureKey, %params);

    $url->query_form_hash(%params, signature => $signature);

    return $url->as_string();
}

=head2 signature($signatureKey, %params) 

Returns just bare signature

=cut 

sub signature {
    my ($signatureKey, %params) = @_;
    my @values = map { $_.'='.(defined $params{$_} ? $params{$_} : "") }
                        sort keys %params;


    my $encString = join(":", $signatureKey, @values);
    utf8::encode($encString);

    return lc(sha1_hex($encString));
}

=head1 Common Functions 

=head2 filter_params( %params )

Returns hash with filtered keys from input hash-reference.

Following keys are preserved: version, shopID, priceAmount, priceCurrency, 
description, referenceID, saleID, custom\w* (all alphanumeric keys begins 
with "custom"). 

Keys with empty values (length <= 0) are removed.

=cut 

sub filter_params {
    my (%params) = @_;

    my @keys = grep { m/ ^(
                            version
                            | shopID
                            | price(Amount|Currency)
                            | description
                            | referenceID
                            | saleID
                            | custom\w*
                        )$ 
                    /x } keys %params;

    my %filtered = map { $_ => $params{$_} } @keys;

    # Delete empty parameters (with string length <= 0)
    foreach (keys %filtered) {
        if (length($filtered{$_}) <= 0) {delete $filtered{$_}};
    }

    return %filtered;
}

1;

__END__



